//=require formily/editor/input_model
//=require formily/editor/input_on_change
//=require formily/editor/editor_buttons
//=require formily/editor/related_fields

(function() {
  var FormilyEditor = function(){

    var sortableInputsList, sortableGroup, formilyModal;

    this.styleCss = 'foundation';

    this.options = {};
    this.content = null;

    this.dialogType = 'input';
    this.actionType = 'add';
    this.element = null;

    // Input object
    this.inputModel = window.InputModel;

    // For sortable placeholder
    var placeholdeElement = function(currentItem) {
      var plh = $('<li></li>');
      plh.css('height', currentItem.css('height'));
      plh.css('margin', currentItem.css('margin'));
      plh.css('border', '1px dotted #000000');
      return plh.get(0);
    };

    this.setSortable = function(){
      sortableInputsList = $('#formily_sortable_inputs');
    };

    this.inputsUpdate = function(){
      // Change index for all inputs
      sortableInputsList.children().each(function(index){
        var self = $(this);
        self.find('[name="inputs[]field_order"]:first').val(index + 1);

        var id_el = $(this).find('span[data-id]:first');
        id_el.text(index + 1);
        id_el.attr('data-id', index + 1);
      });
    };

    this.inputsGroupUpdate = function(target){
      var self, id_el;
      target.children().each(function(index){
        self = $(this);
        self.find('[name="inputs[]group_inputs[]field_order"]:first').val(index + 1);
        id_el = self.find('span[data-id]');
        id_el.text(index + 1);
        id_el.attr('data-id', index + 1);
      });
    };

    this.getHtmlTemplate = function(){
      var list, callbackFun;
      var d = window.InputModel.getBaseObj();

      if(window.FormilyEditor.dialogType == 'group') {
        list = window.FormilyEditor.element;
        callbackFun = window.FormilyEditor.inputsGroupUpdate;
        d.group_input = true;
        d.group_input_id = window.InputModel.group_input_id;
      } else {
        list = sortableInputsList;
        callbackFun = window.FormilyEditor.inputsUpdate;
      }

      //create
      delete(d.id);
      d.form_id = $('input[type="hidden"][name="formily_form[id]"]').val();
      $.post(window.FormilyRoutes.input_create_path(), d)
        .success(function(data){
          d.id = data.id;
          $.post('/formily/inputs/render_input_html', d)
            .done(function(data){
                list.append(data);
                callbackFun(list);
            });
        });
    };

    this.clearDialog = function(modal){
      // Clean all fields
      modal.find('[data-ff-name][type=text]').val('');
      modal.find('textarea[data-ff-name]').val('');
      modal.find('[data-ff-name][type=checkbox]').removeAttr('checked');
      modal.find('select[data-ff-name] option').removeAttr('selected');
      modal.find('section[data-formily="Formily::TextInput"] input[type="text"][data-ff-name="default_value"]').hide(0);
      modal.find('ul.related_value_tag').css('display', 'none');
      var tags = modal.find('ul.related_value_tag');
      tags.tagit('removeAll');
      window.FormilyEditor.options = {};
    };

    // Event on dialog show
    this.onDialogShow = function(event, ui){

      var model = window.InputModel.getBaseObj();

      formilyModal.find('section').hide(0);
      formilyModal.find('section[data-formily="' + window.FormilyEditor.dialogType + '"]').show(0);
      var section = formilyModal.find('section[data-formily="' + model.type + '"]');
      section.show(0);

      if(window.FormilyEditor.dialogType == 'input') {
        var select = formilyModal.find('select[data-ff-name=type]');

        if(select.length > 0){
          select.find('option').removeAttr('selected');
          var option = select.find("option[value='" + model.type + "']");

          if(option.length > 0) {option.get(0).selected = true;}

          if(window.FormilyEditor.actionType == 'edit'){
            select.attr('disabled', 'disabled')
          } else {
            select.removeAttr('disabled')
          }
        }
      } else {
        section = formilyModal.find('section[data-formily="' + window.FormilyEditor.dialogType + '"]');
        section.show(0);
        section = section.find('section[data-formily="' + model.type + '"]');
        section.show(0);
      }

      // Initialize text box value
      section.find('input[data-ff-name], textarea[data-ff-name]').not('[type=checkbox]').each(function(_, elem){
        var self = $(elem);
        self.val(model[self.data('ffName')])
      });

      section.find('select[data-ff-name]').each(function(_, elem){
        var self = $(elem);

        if(self.data('ffName') != 'type') {
          self.find('option').removeAttr('selected');

          if (self.data('ffName') == 'tag_attributes') {
            var option = self.find("option[value='" + window.InputModel.getTagAttr()[self.data('ffSname')] + "']");
            if(option.length > 0) {option.get(0).selected = true;}
          } else {
            var option = self.find("option[value='" + model[self.data('ffName')] + "']");
            var input = self.parent().find('input[type=text][data-ff-name=default_value]:first');
            if(option.length > 0) {
              option.get(0).selected = true;
              input.hide(0);
            }

            if(window.InputModel.default_value == 'custom_value') {
              input.show(0);
            }

            if(window.InputModel.default_value == 'related_value') {
              var tag = self.parents('section:first').find('ul.related_value_tag');
              tag.show();

              var tagRelated = window.InputModel.getTagAttr()['data-related'];

              if(tagRelated && tagRelated.length > 0) {
                var tagArr = tagRelated.split('.');

                tagArr.forEach(function(item){
                  tag.tagit('createTag', item.toString(), 'fake');
                });
              }
            }

            if(self.data('ffName') == 'default_value' && option.length == 0) {
              // Custom value
              input.show(0);
              input.val(window.InputModel.default_value);
              option = self.find('option[value=custom_value]');
              if(option.length > 0) {option.get(0).selected = true;}
            }
          }

        }

      });

      section.find('input[type=checkbox][data-ff-name]').each(function(_, elem){
        var self = $(elem);

        if(self.data('ffName') != 'type') {
          if (self.data('ffName') == 'tag_attributes') {

            if(window.InputModel.getTagAttr()[self.data('ffSname')] != undefined) {
              self.get(0).checked = true;
            } else {
              self.get(0).checked = false;
            }

          } else {
            if(window.InputModel[self.data('ffName')].toString().length > 0) {
              self.get(0).checked = true;
            } else {
              self.get(0).checked = false;
            }
          }
        }

      });

    };

    this.buttonOkClick = function(){

      var model = window.InputModel.getBaseObj();

      if(model.name == '' && model.label == '' && model.type != 'Formily::StaticText') {

        alert("Name can't be blank !");

      } else {

        if(model.name == '') {
          model.name = model.label;
          window.InputModel.name = model.label;
        }

        if(window.FormilyEditor.actionType == 'edit'){

          var elem = window.FormilyEditor.element;
          var d = window.InputModel.getBaseObj();

          $.ajax({
            url: FormilyRoutes.input_update_path(d.id),
            data: d,
            type: 'PUT'
          });

          elem.find('fieldset:first').find('input').each(function () {
            $(this).val(model[$(this).attr('name').replace(/inputs\[\](group_inputs\[\])?/i, '')]);
          });

          elem.find('.ff-input-data-container span[data-input="name"] i:first').text(model.name);
          elem.find('.ff-input-data-container span[data-input="description"] i:first').text(model.description);
          elem.find('.ff-input-data-container span[data-input="label"] i:first').text(model.label);

        } else {
          if(window.FormilyEditor.dialogType == 'input') {
            sortableInputsList.append(window.FormilyEditor.getHtmlTemplate());
          } else {
            window.FormilyEditor.element.append(window.FormilyEditor.getHtmlTemplate());
          }

        }

        setTimeout(function(){$(document).trigger('formily_form_updated');}, 1000);

        $(this).dialog("close");
      }

    };

    this.buttonCloseClick = function(){
      $(this).dialog("close");
    };

    this.setModal = function(){

      // Create modal dialog
      formilyModal = $("#formily_modal").dialog({
        autoOpen: false,
        title: 'Modal',
        resizable: false,
        draggable: false,
        height: 700,
        width: 600,
        create: function(event, ui) {
          var dialog = $(event.target).parents('.ui-dialog');

          dialog.addClass('formily-ui');

          if(window.FormilyEditor.styleCss === 'foundation') {
            //fix button style
            dialog.find('.ui-dialog-buttonpane button.ui-button').attr('class', 'button');
            dialog.find('.ui-dialog-buttonset').attr('class', '')
          }

        },
        close: function(event, ui) {
          window.FormilyEditor.clearDialog($(this));
        },
        show: {
          effect: "size",
          duration: 400
        },
        hide: {
          effect: "size",
          duration: 400
        },
        open: window.FormilyEditor.onDialogShow,
        buttons: [
          {
            text: "Ok",
            click: window.FormilyEditor.buttonOkClick
          },
          {
            text: "Close",
            click: window.FormilyEditor.buttonCloseClick
          }
        ]
      });

      // JQuery UI fix
      $(document).on('mouseover mousedown', 'button', function() {
        $(this).removeClass("ui-state-hover");
        $(this).removeClass("ui-state-active");
        $(this).blur();
        $(this).off('focus');
      });
    };

    this.initialize = function() {
      this.setSortable();
      this.setModal();
    };
  };

  window.FormilyEditor = new FormilyEditor();

})();
