# coding: utf-8
class Formily::InputsController < FormilyController

  def create
    input = Formily::Input.create(input_params)
    render json: {id: input.id}, status: :ok
  end

  def update
    input = Formily::Input.find(params[:id])
    input.update(input_params)
    render nothing: true, status: :ok
  end

  def destroy
    input = Formily::Input.find(params[:id])

    count = Formily::InputValue.where(input_id: input.id).count

    if input.inputs.count > 0
      count += Formily::InputValue.where(input_id: input.inputs.map(&:id)).count
    end

    if input.type == 'Formily::RadioInput'
      count += Formily::InputValue.where(input_id: input.group_input_id).count
    end

    if input.type == 'Formily::OptionInput'
      count += Formily::InputValue.where(input_id: input.group_input_id).count
    end

    if count > 0
      input.update(is_deleted: true)
    else
      input.destroy
    end

    render nothing: true, status: :ok
  end

  def render_input_html
    if params[:type].present?
      @input = params[:type].constantize.new(params.permit(:id, :name, :label, :description, :type,
                                                           :field_order, :tag_attributes, :default_value))
      render layout: false
    end
  end

  def render_related

    if params[:related_arr]

      if params[:related_arr].blank?
        models = Formily.formily_models
        data = []

        models.each do |model|
          data << {label: model.to_s, value: model.to_s}
          data << {
            label: model.to_s + '->' + model.formily_form_related_model.to_s,
            value: model.to_s + '->' + model.formily_form_related_model.to_s
          }
        end

        render json: data.to_json, status: :ok

      else

        model = if params[:related_arr][0] =~ /->/
                  params[:related_arr][0].sub(/.*?->/, '').safe_constantize
                else
                  params[:related_arr][0].safe_constantize
                end

        if model
          tmp = model.new

          params[:related_arr][1..-1].each do |attr|
            tmp = tmp.send('build_' + attr) rescue nil
            tmp = tmp.send(attr) rescue nil unless tmp
          end

          if tmp && tmp.class.superclass.name =~ /\Aactiverecord/i
            model = tmp.class
            data = []
            if model.reflect_on_all_associations.any?
              model.reflect_on_all_associations.each do |ref|
                data << {label: ref.name, value: ref.name, macro: ref.macro} if ref.macro =~ /belongs_to/i
              end
            end
            data += model.columns_hash.map{|k,v| {label: k, value: k}}

            render json: data.to_json, status: :ok
          else
            render text: '', status: :ok
          end

        end
      end

    else
      render text: '', status: :ok
    end

  end

  private
    def input_params
      params.permit(:id, :name, :type, :label, :description, :group_input_id,
                    :field_order, :default_value, :tag_attributes, :form_id)
    end

end