# coding: utf-8
class Formily::Form < ActiveRecord::Base

  after_destroy :delete_data
  before_create :set_default

  self.table_name = 'formily_forms'

  has_paper_trail

  belongs_to :form_type, foreign_key: :form_type_id, class_name: 'Formily::FormType'
  has_many :inputs, foreign_key: :form_id, class_name: 'Formily::Input'
  has_many :submits, foreign_key: :form_id, class_name: 'Formily::Submit'

  def form_inputs
    self.inputs.where('group_input_id IS NULL')
  end

  def get_submits(obj, user_id)
    Formily::Submit.where(form_id: self.id, resource_id: obj.id,
                          user_id: user_id, resource_type: obj.class.name).order(created_at: :desc)
  end

  def response_url(*args)
    '#response'
  end

  def find_or_create_submission_by(attributes)
    submission = submits.find_by(id: attributes[:submit_id])
    submission ||= submits.find_by(resource_id: attributes[:model_id], user_id: attributes[:user_id], resource_type: attributes[:model_name])
    submission ||= submits.create(resource_id: attributes[:model_id], user_id: attributes[:user_id], resource_type: attributes[:model_name])
  end

  private

    def delete_data
      self.inputs.destroy_all
      self.submits.destroy_all
      Formily::FormilyableForm.where(form_id: self.id).destroy_all
    end

    def set_default
      self.is_deleted = false if self.is_deleted.nil?
      true # Fix
    end

end
