# coding: utf-8
class Formily::Input < ActiveRecord::Base

  after_destroy :delete_data
  before_create :set_default

  self.table_name = 'formily_inputs'

  has_paper_trail

  has_many :data, foreign_key: :input_id, class_name: 'Formily::InputValue'
  belongs_to :form, foreign_key: :form_id, class_name: 'Formily::Form'
  # Query fix relation
  has_many :inputs, foreign_key: :group_input_id, class_name: 'Formily::Input'
  belongs_to :group_input, foreign_key: :group_input_id, class_name: 'Formily::Input'

  default_scope ->{ order(:field_order) }

  def render_attributes
    tag_attr = JSON.parse(self.tag_attributes) rescue {}

    if tag_attr['formily_conditioned'].present?
      tag_attr.delete('formily_conditioned')
    end

    tag_attr.map{|k, v| "#{k}=\"#{v}\""}.join(' ').html_safe
  end

  def render_attributes_hash
    JSON.parse(self.tag_attributes).except('formily_conditioned') rescue {}
  end

  def conditioned_tag
    tag_attr = JSON.parse(self.tag_attributes) rescue {}

    if tag_attr['formily_conditioned'].present?
      tag_attr['formily_conditioned']
    else
      ''
    end
  end

  def partial_path
    'formily/partial/inputs/' + self.class.name.demodulize.underscore
  end

  def to_html
    "<label>#{self.label}<input type=\"text\" name=\"formily_inputs[#{self.name}]\"/></label>"
  end

  def editor_type
    'input'
  end

  def get_value_from_submit(submit, controller = nil)
    val = if submit.present?
            submit.input_values.where(input_id: self.id).pluck(:input_value).first || nil rescue nil
          else
            nil
          end


    get_default_value(val, controller)
  end

  def get_default_value(val, controller)

    if val.blank?
      if self.default_value.present?

        case self.default_value

          when 'Date'
            DateTime.now.strftime('%Y/%m/%d')
          when 'DateTime'
            DateTime.now.strftime('%Y/%m/%d %H:%M')
          when 'current_user_name'
            controller.current_user.name rescue ''
          when 'device_location'
            'device_location'
          when 'related_value'
            tag_attr = JSON.parse(self.tag_attributes) rescue {}
            val_arr = tag_attr['data-related'].split('.')

            @obj_rel = if val_arr.first =~ /->/
                         m = val_arr.first.sub(/.*?->/, '').safe_constantize
                         user_id = controller.params["user_id"]
                         if user_id
                           m.find(user_id)
                         else
                           nil
                         end
                       else
                         m = controller.instance_variable_get('@obj_rel')
                         m || val_arr.first.safe_constantize
                       end

            val_arr[1..-1].inject(@obj_rel){|obj, t| obj.send(t) } rescue ''

          else
            self.default_value
        end

      else
        self.tag_attributes =~ /"readonly":"readonly"/ ? self.label : val
      end
    else
      val
    end
  end

  private

    def delete_data
      self.inputs.destroy_all
      self.data.destroy_all
    end

    def set_default
      self.is_deleted = false if self.is_deleted.nil?
      true # Fix
    end

end
